@@ -9,19 +9,13 @@ module Data.Containers where
 
 import Prelude hiding (lookup)
 import Data.Maybe (fromMaybe)
-#if MIN_VERSION_containers(0, 5, 0)
 import qualified Data.Map.Strict as Map
 import qualified Data.IntMap.Strict as IntMap
-#else
-import qualified Data.Map as Map
-import qualified Data.IntMap as IntMap
-#endif
 import qualified Data.HashMap.Strict as HashMap
 import Data.Hashable (Hashable)
 import qualified Data.Set as Set
 import qualified Data.HashSet as HashSet
 import Data.Monoid (Monoid (..))
-import Data.Semigroup (Semigroup)
 import Data.MonoTraversable (MonoFunctor(..), MonoFoldable, MonoTraversable, Element, GrowingAppend, ofoldl', otoList)
 import Data.Function (on)
 import qualified Data.List as List
@@ -67,9 +61,7 @@ class (Data.Monoid.Monoid set, Semigroup set, MonoFoldable set, Eq (ContainerKey
     -- | Get a list of all of the keys in the container.
     keys :: set -> [ContainerKey set]
 
-#if MIN_VERSION_containers(0, 5, 0)
 -- | This instance uses the functions from "Data.Map.Strict".
-#endif
 instance Ord k => SetContainer (Map.Map k v) where
     type ContainerKey (Map.Map k v) = k
     member = Map.member
@@ -87,9 +79,7 @@ instance Ord k => SetContainer (Map.Map k v) where
     keys = Map.keys
     {-# INLINE keys #-}
 
-#if MIN_VERSION_containers(0, 5, 0)
 -- | This instance uses the functions from "Data.HashMap.Strict".
-#endif
 instance (Eq key, Hashable key) => SetContainer (HashMap.HashMap key value) where
     type ContainerKey (HashMap.HashMap key value) = key
     member = HashMap.member
@@ -107,9 +97,7 @@ instance (Eq key, Hashable key) => SetContainer (HashMap.HashMap key value) wher
     keys = HashMap.keys
     {-# INLINE keys #-}
 
-#if MIN_VERSION_containers(0, 5, 0)
 -- | This instance uses the functions from "Data.IntMap.Strict".
-#endif
 instance SetContainer (IntMap.IntMap value) where
     type ContainerKey (IntMap.IntMap value) = Int
     member = IntMap.member
@@ -213,9 +201,7 @@ class PolyMap map where
     intersectionWithMap :: (value1 -> value2 -> value3)
                         -> map value1 -> map value2 -> map value3
 
-#if MIN_VERSION_containers(0, 5, 0)
 -- | This instance uses the functions from "Data.Map.Strict".
-#endif
 instance Ord key => PolyMap (Map.Map key) where
     differenceMap = Map.difference
     {-# INLINE differenceMap #-}
@@ -225,9 +211,7 @@ instance Ord key => PolyMap (Map.Map key) where
     intersectionWithMap = Map.intersectionWith
     {-# INLINE intersectionWithMap #-}
 
-#if MIN_VERSION_containers(0, 5, 0)
 -- | This instance uses the functions from "Data.HashMap.Strict".
-#endif
 instance (Eq key, Hashable key) => PolyMap (HashMap.HashMap key) where
     differenceMap = HashMap.difference
     {-# INLINE differenceMap #-}
@@ -237,9 +221,7 @@ instance (Eq key, Hashable key) => PolyMap (HashMap.HashMap key) where
     intersectionWithMap = HashMap.intersectionWith
     {-# INLINE intersectionWithMap #-}
 
-#if MIN_VERSION_containers(0, 5, 0)
 -- | This instance uses the functions from "Data.IntMap.Strict".
-#endif
 instance PolyMap IntMap.IntMap where
     differenceMap = IntMap.difference
     {-# INLINE differenceMap #-}
@@ -568,9 +550,7 @@ class (MonoTraversable map, SetContainer map) => IsMap map where
     filterMap :: IsMap map => (MapValue map -> Bool) -> map -> map
     filterMap p = mapFromList . filter (p . snd) . mapToList
 
-#if MIN_VERSION_containers(0, 5, 0)
 -- | This instance uses the functions from "Data.Map.Strict".
-#endif
 instance Ord key => IsMap (Map.Map key value) where
     type MapValue (Map.Map key value) = value
     lookup = Map.lookup
@@ -619,9 +599,7 @@ instance Ord key => IsMap (Map.Map key value) where
     filterMap = Map.filter
     {-# INLINE filterMap #-}
 
-#if MIN_VERSION_containers(0, 5, 0)
 -- | This instance uses the functions from "Data.HashMap.Strict".
-#endif
 instance (Eq key, Hashable key) => IsMap (HashMap.HashMap key value) where
     type MapValue (HashMap.HashMap key value) = value
     lookup = HashMap.lookup
@@ -658,9 +636,7 @@ instance (Eq key, Hashable key) => IsMap (HashMap.HashMap key value) where
     filterMap = HashMap.filter
     {-# INLINE filterMap #-}
 
-#if MIN_VERSION_containers(0, 5, 0)
 -- | This instance uses the functions from "Data.IntMap.Strict".
-#endif
 instance IsMap (IntMap.IntMap value) where
     type MapValue (IntMap.IntMap value) = value
     lookup = IntMap.lookup
@@ -703,10 +679,8 @@ instance IsMap (IntMap.IntMap value) where
     {-# INLINE unionsWith #-}
     mapWithKey = IntMap.mapWithKey
     {-# INLINE mapWithKey #-}
-#if MIN_VERSION_containers(0, 5, 0)
     omapKeysWith = IntMap.mapKeysWith
     {-# INLINE omapKeysWith #-}
-#endif
     filterMap = IntMap.filter
     {-# INLINE filterMap #-}
 
